/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef _CALLBACKS_H
#define _CALLBACKS_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/encoding.h>
#include <libxml/xmlwriter.h>
	
/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#define SEPAR '|'

#define rackMax   		99
#define rackRowMax  	99
#define rackColMax	99

#define Size_DBPATH 		80
#define Size_CONF   		80

#define date_size        	4

#define domain_size        	64
// general
#define supplier_size 	64
#define supplierAdr_size      128
#define vintage_size   	4
#define color_size    	32
#define country_size               32
#define region_size     	32
#define appellation_size     	64
#define cepage_size          	128
#define potentielDeGarde_size    	date_size 
// stock
#define emplacement_size        32
#define rang_size               8
#define temperature_size        10
#define qty_size       		8 
// achat
#define purchaseDate_size      	date_size
#define price_size       	12
// consommation
#define drinkingDate_size	date_size 
#define drinkingRated_size          4
#define drinkingComment_size   128
// commentaire
#define comment_size	1024
#define rated_size       	4 

#define print_entete "| REGION     | APPELLATION       | NOM                            |MILE.|QTE.|\n"
#define print_corps  "|            |                   |                                |     |    |\n"
#define print_ligne_separ "------------------------------------------------------------------------------\n"
#define print_colonne_separ " | "

enum
{
	COL_TREEVIEW,
	NUM_COLS_TREEVIEW
} ;

enum
{
	COL_TREELIST_MILLESIME,
	COL_TREELIST_QTY,
	COL_TREELIST_DOMAIN,
	COL_TREELIST_COUNTRY,
	COL_TREELIST_REGION,
	COL_TREELIST_APPELLATION,
	COL_TREELIST_COLOR,
	COL_TREELIST_MATURITE,
	COL_TREELIST_PDATA,
	COL_TREELIST_PGLIST,
	NUM_COLS_TREELIST
} ;

enum
{
	COL_TREELIST_PURCHASE_DATE,
	COL_TREELIST_PURCHASE_QTY,
	COL_TREELIST_PURCHASE_PRICE,
	COL_TREELIST_SUPPLIER,
	COL_TREELIST_SUPPLIER_ADR,
	NUM_COLS_TREELIST_PURCHASE
};
enum
{
	COL_TREELIST_DRINKING_DATE,
	COL_TREELIST_DRINKING_QTY,
	COL_TREELIST_NOTE,
	COL_TREELIST_COMMENTAIRE,
	NUM_COLS_TREELIST_DRINKING
};

/*
 * ********************************************************
 * variables issues du fichier de configuration ~/.xcaverc
 */
gchar DBPATH[Size_DBPATH];
gchar DBVERS[24];
gchar HEIGHT[8];
gchar WIDTH[8];
gchar fileLocation[Size_CONF];
gchar printerLocation[Size_CONF];
gchar browserLocation[Size_CONF];

/*
 * dimension de la fenetre Detail
 */
gchar WindowsDetailHeight[8];
gchar WindowsDetailWidth[8];
/*
 * dimension de la fenetre rack
 */
gchar WindowsRackHeight[8];
gchar WindowsRackWidth[8];
/*
 * 0 pas d'affichage des vins ayant une quantite nulle
 * 1 affichage de tous les vins
 */
gint  QT;
/*
 */
gint  DetailMaxRowEmplacement;
gint  DetailMaxRowPurchase;
gint  DetailMaxRowDrinking;
/*
 * structure de l'emplacement de stockage
 */
gint  rackNb;
gint  rackRowNb;
gint  rackColNb;
/*
 * ***********************************************************
 */

/*
 * structure de donnees pour les g_list
 */
typedef struct {
		gchar 		Nom[domain_size];
		gchar 		vintage[vintage_size];
		gchar 		supplier[supplier_size];     //supplier
		gchar 		qty[qty_size];
	} Detail;
typedef Detail *ptrDetail;

typedef struct {
		gchar      country[country_size];
		gchar 		Region[region_size];
		gchar 		Appellation[appellation_size];
		gchar 		color[color_size];
	} Categ;
typedef Categ *ptrCateg;

typedef struct {
	gchar 		Region[region_size];
	gchar 		Appellation[appellation_size];
	} Appellation;
typedef Appellation *ptrAppellation;

typedef struct {
		gchar       country[country_size];
		gchar 		Region[region_size];
	} Region;
typedef Region *ptrRegion;

typedef struct {
		gint		Rack;
		gint		Row;
		gint		Column;
		gchar 		Nom[domain_size];
	} Emplacement;
typedef Emplacement *ptrEmplacement;

typedef struct {
		gint		Rack;
		gint		Row;
		gint		Column;
		gchar 		Nom[domain_size];
		gpointer	ptr_list_stock_data;
		gpointer	ptr_widget_name;		
		// GtkWidget	*widget_name;		
	} Rack;
typedef Rack *ptrRack;

typedef struct {
		gint 	Achat;
		gint	qty;
		gfloat	Prix;
		gchar   Supplier[supplier_size + 1];
		gchar   SupplierAdr[supplierAdr_size + 1];
	} Purchase;
typedef Purchase *ptrPurchase;

typedef struct {
		gint	Conso;
		gint	qty;
		gchar 	Note[rated_size + 1];
		gchar 	Commentaire[drinkingComment_size + 1];
	} Drinking;
typedef Drinking *ptrDrinking;

typedef struct {
		gint		qty;
		GList     *list_purchase;
	} purchase_info;
typedef purchase_info *ppurchase_info;

typedef struct {
		  gint		qty;
		  GList     *list_drinking;
	} drinking_info;
typedef drinking_info *pdrinking_info;

// structure de l'enregistrement  partir de la version 1.1
typedef struct {
		// general
		gchar		country[country_size];
		gchar 		Region[region_size];
		gchar 		Appellation[appellation_size];
		gchar 		Nom[domain_size];
		gint 		vintage;
		gchar 		color[color_size];
		gchar 		supplier[supplier_size];     //supplier
		gchar 		supplierURL[supplierAdr_size];
		gchar 		Cepage[cepage_size];
		gint 		Conserv;		      // dure de conservation
		gchar       temperatureDeDegustation[temperature_size];
		// stock
		gint 		qty;
		gint		NbEmplacement;
		GList       *list_emplacement;
		// Achat  - Purchase	
		gint		NbPurchase;
		GList       *list_purchase;
		// Consommation - Drinking	
		gint		NbDrinking;
		GList       *list_drinking;
		// Commentaire
		gchar 		Commentaire[comment_size];
		gchar 		Note[rated_size];
		// Global
		gint        Maturite;  // taux de maturit : champs calcul (=now - Achat / Conserv)

	} Vin;
typedef Vin *ptrVin;

// structure xml
/*
 * A xmlRacks record
 * an xmlChar * is really an UTF8 encoded char string (0 terminated)
 */
typedef struct xmlRacks {
    xmlChar *rackNb;
    xmlChar *row;
    xmlChar *column;
} xmlRacks, *xmlRacksPtr;

/*
 * A xmlDrinking record
 * an xmlChar * is really an UTF8 encoded char string (0 terminated)
 */
typedef struct xmlDrinking {
    xmlChar *date;
    xmlChar *qty;
    xmlChar *rated;
    xmlChar *comment;
} xmlDrinking, *xmlDrinkingPtr;

/*
 * A xmlPurchase record
 * an xmlChar * is really an UTF8 encoded char string (0 terminated)
 */
typedef struct xmlPurchase {
    xmlChar *date;
    xmlChar *qty;
    xmlChar *price;
    xmlChar *supplier;
    xmlChar *supplierAdr;
} xmlPurchase, *xmlPurchasePtr;

typedef struct xmlWine {
    xmlChar *country;
    xmlChar *region;
    xmlChar *appellation;
    xmlChar *domain;
    xmlChar *vintage;
    xmlChar *color;
    xmlChar *wineMaker;
    xmlChar *wineMakerAdr;
    xmlChar *cepage;
    xmlChar *potentielDeGarde;
    xmlChar *temperatureDeDegustation;
    int nbRacks;
    xmlRacksPtr xmlRacks[100];
    int nbPurchase;
    xmlPurchasePtr xmlPurchase[100];
    int nbDrinking;
    xmlDrinkingPtr xmlDrinking[100];
    xmlChar *rated;
    xmlChar *comment;
} xmlWine, *xmlWinePtr;


gint nowan;
gint NbBottleStock;  /* compteur 'Nombre de bouteille dans le stock */
gint NbBottleDetail; /* compteur 'Nombre de bouteille detail (ie d'un meme vin) */

/* Flag indiquant l'origine de la fenetre detail */
gint window_detail_origine;
enum {
	NO_BUTTON = 0,
	BUTTON_ADD = 1,
	BUTTON_MODIF = 2,
	BUTTON_RACK = 3,
	BUTTON_RACK_ADD = 4,
	BUTTON_RACK_MODIF = 5
};

GList  *list_stock;

GList  *list_categ;
ptrCateg Ctg;

GList	*list_country;
GList	*list_region;
GList	*list_country_region;
GList	*list_appellation;
GList	*list_region_appellation;
GList	*list_cepage;
GList	*list_rack;

GdkFont		*fixed;

GtkWidget	*window_xcave ;		/* Main window */
GtkWidget	*window_detail;
GtkWidget	*window_about;
GtkWidget	*confirm;
GtkWidget	*window_print;
GtkWidget	*window_sommelier;
GtkWidget	*window_config;
GtkWidget	*window_fileselection_stock;
GtkWidget	*window_rack;
GtkWidget 	*window_dialog;
guint    signal_id_app;
guint    signal_id_reg;
guint    signal_id_pay;

gint statid_stock;		/* barre d'tat */
guint ctxtid_stock;
gint statid_detail;		/* barre d'tat */
gint ctxtid_detail;

typedef enum {
	erreur = 0,
	existe = 1,
	nouvel = 2
} ecateg;

gint flagprint;
typedef enum { 
	lpr = 0, 
	file = 1,
	browser = 2
} eprinter;

int flagrecord;
typedef enum { 
	all = 0, 
	record = 1
} erecord;

#include "callbacks_proto.h"

void
on_TreeList_row_activated              (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data);

void
on_TreeView_cursor_changed             (GtkTreeView     *treeview,
                                        gpointer         user_data);

void
on_modif_button_clicked                (GtkButton       *button,
                                        gpointer         user_data);

void
on_supp_button_clicked                 (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_detail_destroy               (GtkObject       *object,
                                        gpointer         user_data);

void
on_window_detail_check_resize          (GtkContainer    *container,
                                        gpointer         user_data);

void
on_window_rack_check_resize          (GtkContainer    *container,
                                        gpointer         user_data);

void
on_button_add_clicked                  (GtkButton       *button,
                                        gpointer         user_data);

void
on_entry_color_changed                 (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_button_add_ok_clicked               (GtkButton       *button,
                                        gpointer         user_data);

void
on_button_add_cancel_clicked           (GtkButton       *button,
                                        gpointer         user_data);

void
maturite_cell_data_function 		(GtkTreeViewColumn *col,
								   GtkCellRenderer   *renderer,
								   GtkTreeModel      *model,
								   GtkTreeIter       *iter,
								   gpointer           user_data);

gboolean
appellation_row_visible_function	(GtkTreeModel	*model,
									 GtkTreePath  *path,
                                     GtkTreeIter 	*iter,
                                     gpointer		data);
void
xcave_fill_filtered_model (	gpointer	data,
							gint		colonne);
								
gint 
xcave_compute_maturity_value(gint	qt, 
							 gint	millesime,
							 gint	conservation);

void
xcave_create_TreeList_purchase(void);

void
xcave_create_TreeList_drinking(void);

ptrVin
xcave_id_selected_treelist_row 			(void);

ptrVin
xcave_id_selected_toggledbutton			(void);

void
on_button_ok_clicked                   (GtkButton       *button,
                                        gpointer         user_data);

gboolean
xcave_list_purchase_append (GtkTreeModel		*model,
						    GtkTreePath		*path,
						    GtkTreeIter		*iter,
						    gpointer			data);

gboolean
xcave_list_drinking_append (GtkTreeModel		*model,
						    GtkTreePath		*path,
						    GtkTreeIter		*iter,
						    gpointer			data);

void
on_window_detail_button_ok_clicked     (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_detail_button_cancel_clicked (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_add_button_clicked     (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_supp_button_clicked    (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_som_button_clicked     (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_modif_button_clicked   (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_button_print_clicked   (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_button_config_clicked  (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_button_quit_clicked    (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_button_about_clicked   (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_supp_button_supp_ok_clicked  (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_supp_button_supp_cancel_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_supp_destroy                 (GtkObject       *object,
                                        gpointer         user_data);

void
on_window_print_destroy                (GtkObject       *object,
                                        gpointer         user_data);

void
on_window_sommelier_destroy            (GtkObject       *object,
                                        gpointer         user_data);

void
on_window_config_destroy               (GtkObject       *object,
                                        gpointer         user_data);

void
on_window_sommelier_button_ok_clicked  (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_config_button_ok_clicked     (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_config_button_cancel_clicked (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_print_button_ok_clicked      (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_print_button_cancel_clicked  (GtkButton       *button,
                                        gpointer         user_data);

void
on_radiobutton_print_all_pressed       (GtkButton       *button,
                                        gpointer         user_data);

void
on_radiobutton_print_this_pressed      (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_print_button_ok_clicked      (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_print_button_cancel_clicked  (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_add_button_clicked     (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_modif_button_clicked   (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_supp_button_clicked    (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_som_button_clicked     (GtkButton       *button,
                                        gpointer         user_data);

/* add 'directory' plus subdirectories recursively to the list
   we browse when looking for files (e.g. pixmaps); */
void
add_support_directory_recursive                   (const gchar     *directory);

/* add 'directory' both to our directory list and
   GLADE's list */
void
add_support_directory(const gchar *directory);

void
on_xcave_window_add_button_clicked     (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_modif_button_clicked   (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_widow_supp_button_clicked     (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_som_clicked            (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_button_print_clicked   (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_button_config_clicked  (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_button_quit_clicked    (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_button_about_clicked   (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_supp_button_clicked    (GtkButton       *button,
                                        gpointer         user_data);

void
on_xcave_window_som_button_clicked     (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_about_destroy                (GtkObject       *object,
                                        gpointer         user_data);

void
on_window_about_button_close_clicked   (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_config_button_file_clicked   (GtkButton       *button,
                                        gpointer         user_data);

void
on_fileselection_stock_destroy         (GtkObject       *object,
                                        gpointer         user_data);

void
on_window_print_destroy                (GtkObject       *object,
                                        gpointer         user_data);

void
on_window_fileselection_stock_ok_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_fileselection_stock_cancel_button1_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data);

void
xcave_fill_model (GtkTreeStore * Treeview, GtkListStore * Treelist);

GList* 
xcave_g_list_free(GList *list); 

void 
xcave_g_list_free_stock(void);

void
on_entry_color_changed                 (GtkEditable     *editable,
                                        gpointer         user_data);
										
void
on_xcave_window_button_rack_clicked    (GtkButton       *button,
                                        gpointer         user_data);

void
on_window_rack_destroy                 (GtkObject       *object,
                                        gpointer         user_data);

void
on_window_rack_button_close_clicked    (GtkButton       *button,
                                        gpointer         user_data);

void
xcave_createRack			(GtkWidget	*window,
							 GList		*list);

GList *
xcave_rackTreatment			(char		*);

void
on_window_rack_toggledbutton_toggled	(GtkToggleButton	*toggledbutton,
										 gpointer         user_data);

void 
list_append_stock ();

void 
list_append_categ ();

void
list_append_appellation ();

void
list_append_region ();

void
list_append_country ();

void
list_append_cepage ();

void 
list_country_append_stock ();

void 
list_region_append_stock ();

void 
list_appellation_append_stock ();

void 
list_append_emplacement ();

void 
list_append_rack ();

void list_cepage_append_stock ();

int cmp_stock(gpointer ,
	  gpointer );

int cmp_region(gpointer ,
           gpointer );

int cmp_nom(gpointer ,
        gpointer );

int cmp_country(gpointer ,
        gpointer );

int cmp_appellation(gpointer ,
        gpointer );

int cmp_categ_nom(gpointer ,
              gpointer );

int cmp_producteur(gpointer ,
	       gpointer );

int cmp_millesime(gpointer ,
	      gpointer );

int cmp_quantite(gpointer ,
	     gpointer );

int cmp_cepage(gpointer ,
	     gpointer );

int find_existing_wine(gpointer ,
           gpointer );

void on_xcave_window_display_modif(Vin	*Vin_rowSelected);

void quick_message (gchar *);

void on_quick_message_okay_button_clicked (gpointer		data,
									  gint			colonne);

void xcave_xmlWrite(const char *file);
int xcave_xmlread(void);

void testXmlwriterMemory(const char *file);

void testXmlwriterDoc(const char *file);

void testXmlwriterTree(const char *file);

void xcave_xmlWriteSingleWine(char *,  xmlTextWriterPtr);

xmlChar *xcave_utf8ToIsolat1(const char *in, const char *encoding);
xmlChar *xcave_isolat1toUtf8(const char *in);


void
on_xcave_window_button_html_clicked    ();

void
on_radiobutton_print_browser_pressed   (GtkButton       *button,
                                        gpointer         user_data);

#endif /* callbacks.h */
