/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"


//************************************ fonctions fichier

FILE *
StocksOpen (char *mode)
{
  FILE *f;

  if (( f = fopen (DBPATH, mode)) == NULL ) {
    if (( f = fopen (DBPATH, "a+")) == NULL ) {
	 fprintf (stderr, _("%s (%d) Open error (%d) file %s\n"),__FILE__,__LINE__,errno,DBPATH);
      exit (-1);
    }
  }

  return (f);
}

void 
StocksFseek(FILE *f)
{
  int cr;

  if (( cr = fseek (f,0,SEEK_SET)) != 0 ) {
      fprintf (stderr, _("%s (%d) Erreur (%d) repositionnement fichier %s\n"),__FILE__,__LINE__,cr,DBPATH);
      exit (-1);
  }
}

void 
StocksFgets (FILE *f, char *pvin)
{
  char *cr;

  // Limite volontaire  20 lignes de type Purchase & drinking
  if ((( cr = fgets (pvin, sizeof (Vin) + 
		                   (DetailMaxRowEmplacement * (sizeof(Emplacement) - domain_size)) + 
		                   (DetailMaxRowPurchase    * (sizeof(Purchase))) + 
			           (DetailMaxRowDrinking    * (sizeof(Drinking))), f)) == NULL ) 
      && 
       (!feof (f))) {
      fprintf (stderr, _("%s (%d) Read error (%d) file %s\n"),__FILE__,__LINE__,errno,DBPATH);
      exit (-1);
    }
}

void 
StocksFputs (char *vindetail,FILE *f)
{
  char             *pvin;
  ptrVin            detail;

  ptrEmplacement    tmpEmplacement;
  GList            *tmp_list_emplacement;

  ptrPurchase       tmpPurchase;
  GList            *tmp_list_purchase;

  ptrDrinking       tmpDrinking;
  GList            *tmp_list_drinking;

#if (defined DEBUG_LEVEL_1)
	printf("StocksFputs:B:%p:\n",(void *)vindetail);
#endif

  detail = (ptrVin) vindetail;

  pvin = malloc ( sizeof(Vin) + 
		    (g_list_length(detail->list_emplacement) * ( sizeof(Emplacement) - domain_size)) + 
		    (g_list_length(detail->list_purchase)    * sizeof(Purchase)) + 
		    (g_list_length(detail->list_drinking)    * sizeof(Drinking)));

  memset(pvin,'\0',sizeof(Vin) +
                   (g_list_length(detail->list_emplacement)  * ( sizeof(Emplacement) - domain_size)) +
                   (g_list_length(detail->list_purchase)     * sizeof(Purchase)) +
                   (g_list_length(detail->list_drinking)     * sizeof(Drinking)));

  char2stocks (pvin, detail->country);
  char2stocks (pvin, detail->Region);
  char2stocks (pvin, detail->Appellation);
  char2stocks (pvin, detail->Nom);
  inte2stocks (pvin, detail->vintage);
  char2stocks (pvin, detail->color);
  char2stocks (pvin, detail->supplier);
  char2stocks (pvin, detail->supplierURL);            // url du producteur
  char2stocks (pvin, detail->Cepage);
  inte2stocks (pvin, detail->Conserv);
  char2stocks (pvin, detail->temperatureDeDegustation);          // temperature de degustation

  // Stock
  inte2stocks (pvin, g_list_length(detail->list_emplacement));

  tmpEmplacement = malloc (sizeof(Emplacement));
  
  for(tmp_list_emplacement = detail->list_emplacement; 
      tmp_list_emplacement != NULL; 
      tmp_list_emplacement = tmp_list_emplacement->next) {

      memset(tmpEmplacement,'\0',sizeof(Emplacement));
      memcpy(tmpEmplacement,tmp_list_emplacement->data,sizeof(Emplacement));

      inte2stocks (pvin, tmpEmplacement->Rack);   
      inte2stocks (pvin, tmpEmplacement->Row);
      inte2stocks (pvin, tmpEmplacement->Column);
  }
  free(tmpEmplacement);

  // Purchase
  inte2stocks (pvin, g_list_length(detail->list_purchase));

  tmpPurchase = malloc (sizeof(Purchase));

  for(tmp_list_purchase = detail->list_purchase; 
      tmp_list_purchase != NULL; 
      tmp_list_purchase = tmp_list_purchase->next) {

      memset(tmpPurchase,'\0',sizeof(Purchase));
      memcpy(tmpPurchase,tmp_list_purchase->data,sizeof(Purchase));

      char2stocks (pvin, tmpPurchase->Supplier);   
      char2stocks (pvin, tmpPurchase->SupplierAdr);
      inte2stocks (pvin, tmpPurchase->Achat);   
      float2stocks (pvin, tmpPurchase->Prix);
      inte2stocks (pvin, tmpPurchase->qty);

  }
  free(tmpPurchase);

  // Drinking
  inte2stocks (pvin, g_list_length(detail->list_drinking));

  tmpDrinking = malloc (sizeof(Drinking));

  for(tmp_list_drinking = detail->list_drinking; 
      tmp_list_drinking != NULL; 
      tmp_list_drinking = tmp_list_drinking->next) {

      memset(tmpDrinking,'\0',sizeof(Drinking));
      memcpy(tmpDrinking,tmp_list_drinking->data,sizeof(Drinking));

      inte2stocks (pvin, tmpDrinking->Conso);   
      inte2stocks (pvin, tmpDrinking->qty);
      char2stocks (pvin, tmpDrinking->Note);
      char2stocks (pvin, tmpDrinking->Commentaire);

  }
  free(tmpDrinking);

  char2stocks (pvin, detail->Commentaire);
  char2stocks (pvin, detail->Note);
 
  strcat (pvin, "\n");

  /*
   * increment des compteurs de lignes Purchase et Drinking pour un malloc optimal au chargement
   */
  if (DetailMaxRowEmplacement < g_list_length(detail->list_emplacement))
      DetailMaxRowEmplacement = g_list_length(detail->list_emplacement);

  if (DetailMaxRowPurchase < g_list_length(detail->list_purchase))
      DetailMaxRowPurchase = g_list_length(detail->list_purchase);

  if (DetailMaxRowDrinking < g_list_length(detail->list_drinking))
      DetailMaxRowDrinking = g_list_length(detail->list_drinking);

  if ( fputs (pvin, f) == EOF ) {
     fprintf (stderr,_("%s (%d) Write error (%d) file %s\n"),__FILE__,__LINE__,errno,DBPATH);
     exit (-1);
  }
  free(pvin);
#if (defined DEBUG_LEVEL_1)
	printf("StocksFputs:E:\n");
#endif
}

void
ReadStocks (FILE *f)
{
  int               i = 0 ;
  int               j = 0 ;
  int               n = 0 ;
  char             *pvin;
  
  ptrEmplacement    tmpEmplacement;
  ptrPurchase       tmpPurchase;
  ptrDrinking       tmpDrinking;
  ptrVin            v;

  pvin = malloc ( sizeof(Vin) + 
	          (DetailMaxRowEmplacement * (sizeof(Emplacement) - domain_size)) + 
	          (DetailMaxRowPurchase    * (sizeof(Purchase))) + 
	          (DetailMaxRowDrinking    * (sizeof(Drinking))));

  memset(pvin, '\0', ( sizeof (Vin) +
                        (DetailMaxRowEmplacement * (sizeof(Emplacement) - domain_size)) +
                        (DetailMaxRowPurchase    * (sizeof(Purchase))) +
                        (DetailMaxRowDrinking    * (sizeof(Drinking)))));

  if ( ( v = g_malloc (sizeof (Vin))) == NULL ) {
       fprintf (stderr, "Probleme allocation memoire.\n");
       exit (-1);
  }

  memset(v,       '\0',  sizeof (Vin));

  StocksFgets (f,pvin);
  if (!feof (f)) {
      stocks2char (pvin, v->country, &i, &n);                     // country d'origine
      stocks2char (pvin, v->Region, &i, &n);                   // region d'origine
      stocks2char (pvin, v->Appellation, &i, &n);              // appellation
      stocks2char (pvin, v->Nom, &i, &n);                      // nom
      stocks2inte (pvin, &v->vintage, &i, &n);               // millesime
      stocks2char (pvin, v->color, &i, &n);                  // couleur
      stocks2char (pvin, v->supplier, &i, &n);               // producteur
      stocks2char (pvin, v->supplierURL, &i, &n);            // url du producteur
      stocks2char (pvin, v->Cepage, &i, &n);                   // cepage
      stocks2inte (pvin, &v->Conserv, &i, &n);                 // nb d'anne de conservation
      stocks2char (pvin, v->temperatureDeDegustation, &i, &n);           // temperature de degustation

      //Stock
      v->qty = 0;
      stocks2inte (pvin, &v->NbEmplacement, &i, &n);  
      for (j=0; j<v->NbEmplacement; j++) {
         tmpEmplacement = malloc (sizeof(Emplacement));
         memset(tmpEmplacement,'\0',sizeof(Emplacement));

         stocks2inte (pvin, &tmpEmplacement->Rack, &i, &n);  // casier
         stocks2inte (pvin, &tmpEmplacement->Row, &i, &n);     // ligne
         stocks2inte (pvin, &tmpEmplacement->Column, &i, &n);  // colonne
	 strcpy(tmpEmplacement->Nom,v->Nom);

	 v->list_emplacement = g_list_append(v->list_emplacement,tmpEmplacement);
      }

      //printf("g_list_length(ReadStocks v->list_emp)=%d\n",g_list_length(v->list_emplacement));

      // Purchase
      stocks2inte (pvin, &v->NbPurchase, &i, &n);
      for (j=0; j<v->NbPurchase; j++) {
         tmpPurchase = malloc (sizeof(Purchase));
         memset(tmpPurchase,'\0',sizeof(Purchase));

         stocks2char (pvin, tmpPurchase->Supplier, &i, &n);        // fournisseur, distributeur, negociant ...
         stocks2char (pvin, tmpPurchase->SupplierAdr, &i, &n);      // url du fournisseur, distributeur, negociant ...
         stocks2inte (pvin, &tmpPurchase->Achat, &i, &n);               // annee d'achat
         stocks2float (pvin, &tmpPurchase->Prix, &i, &n);                // prix
         stocks2inte (pvin, &tmpPurchase->qty, &i, &n);                 // annee d'achat

	 v->list_purchase = g_list_append(v->list_purchase,tmpPurchase);

	 v->qty += tmpPurchase->qty;
      }
      
      // Drinking
      stocks2inte (pvin, &v->NbDrinking, &i, &n);              // nb.achat 

      for (j=0; j<v->NbDrinking; j++) {
         tmpDrinking = malloc (sizeof(Drinking));
         memset(tmpDrinking,'\0',sizeof(Drinking));

         stocks2inte (pvin, &tmpDrinking->Conso, &i, &n);         // fournisseur, distributeur, negociant ...
         stocks2inte (pvin, &tmpDrinking->qty, &i, &n);                 // annee d'achat
         stocks2char (pvin, tmpDrinking->Note, &i, &n);      // url du fournisseur, distributeur, negociant ...
         stocks2char (pvin, tmpDrinking->Commentaire, &i, &n);               // annee d'achat

	 v->list_drinking = g_list_append(v->list_drinking,tmpDrinking);

	 v->qty -= tmpDrinking->qty;
      }

      stocks2char (pvin, v->Commentaire, &i, &n);              // commentaire
      stocks2char (pvin, v->Note, &i, &n);                     // note degustation

      // Calcul du taux de maturit
      if (v->qty == 0) v->Maturite = -2;
      else
          if (v->vintage ==0 || v->Conserv == 0) v->Maturite = -1;
          else
              v->Maturite=(100*(nowan-(v->vintage))/v->Conserv);
      
      list_stock = g_list_append(list_stock ,v);
  }
  free(pvin);
}

void
SaveStocks ()
{
  FILE *f;

  //
  /* Sauvegarde du stock
   */
  f = StocksOpen ("w");

  g_list_foreach(list_stock, (GFunc)StocksFputs, f);
  
  fclose (f);

}
