/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"


void
on_xcave_window_button_config_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{

  FILE *f;
  char filename[256];

	/* 
	 * desactive window_xcave
	 */
	gtk_widget_set_sensitive(window_xcave, FALSE);

	window_config = create_window_config();
	
	memset(filename,'\0',sizeof(filename));
	sprintf(filename,"%s/.xcaverc",getenv("HOME"));
	if ((f = fopen (filename, "r")) == NULL) {
		if ((f= fopen ("/etc/xcave.conf", "r")) == NULL) {
			quick_message (_(" Configuration file error "));
		}
	}
	
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_config, "PathStock")), (gchar *) DBPATH);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_config, "ConfVers")), (gchar *) DBVERS);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_config, "ConfLengthentry")), (gchar *) HEIGHT);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_config, "ConfWidthentry")), (gchar *) WIDTH);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfQt")), (gint) QT);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfNbRack")), (gint) rackNb);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfNbRow")), (gint) rackRowNb);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfNbCol")), (gint) rackColNb);
	gtk_widget_show (window_config);
}


void
on_window_config_button_ok_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkSpinButton *spinqt;
  int lqt = QT;

#if (defined DEBUG_LEVEL_1)
	printf("on_window_config_button_ok_clicked:B:\n");
#endif

  strcpy (DBPATH, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_config, "PathStock"))));
  strcpy (DBVERS, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_config, "ConfVers"))));
  strcpy (HEIGHT, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_config, "ConfLengthentry"))));
  strcpy (WIDTH,  gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_config, "ConfWidthentry"))));
  spinqt = GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfQt"));
  QT              = (int) gtk_spin_button_get_value_as_int (spinqt);
  spinqt          = GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfNbRack"));
  rackNb        = (int) gtk_spin_button_get_value_as_int (spinqt);
  spinqt          = GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfNbRow"));
  rackRowNb     = (int) gtk_spin_button_get_value_as_int (spinqt);
  spinqt          = GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfNbCol"));
  rackColNb  = (int) gtk_spin_button_get_value_as_int (spinqt);

  gtk_object_destroy (GTK_OBJECT (window_config));

   /* 
    * reactive window_xcave
    */
   gtk_widget_set_sensitive(window_xcave, TRUE);

  /*
   * Mise a jour et affichage de l'arbre (country/region/appellation) si le parametre a change
   */
  if ( lqt != QT ) {
		/*
		 * need to update storeview and storelist
		 */
		GtkWidget		*treelist;
		GtkWidget		*treeview;
		GtkListStore	*storelist;
		GtkTreeStore	*storeview;
	
		treelist = lookup_widget(window_xcave,"TreeList");
		storelist	= (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)treelist);
		gtk_list_store_clear(storelist);

		treeview = lookup_widget(window_xcave,"TreeView");
		storeview	=(GtkTreeStore *)  gtk_tree_view_get_model((GtkTreeView *)treeview);
		gtk_tree_store_clear(storeview);
	    
		xcave_fill_model (storeview, storelist);

  }

  /*
   * Resizing de la fenetre pour prise en compte des nouvelles valeurs
   */
  gtk_window_set_default_size (GTK_WINDOW (window_xcave), atoi(WIDTH), atoi(HEIGHT));

	
#if (defined DEBUG_LEVEL_1)
	printf("on_window_config_button_ok_clicked:E:\n");
#endif

}


void
on_window_config_button_cancel_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{

  gtk_object_destroy (GTK_OBJECT (window_config));
  
   /* 
    * reactive window_xcave
    */
   gtk_widget_set_sensitive(window_xcave, TRUE);
}

void
on_window_config_destroy               (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_object_destroy (GTK_OBJECT (window_config));

   /* 
    * reactive window_xcave
    */
   gtk_widget_set_sensitive(window_xcave, TRUE);

}
