/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

void
xcave_create_TreeList_purchase(void)
{
	GtkListStore		*StoreList_purchase;
	GtkWidget			*scrolledwindow_purchase;
	GtkWidget			*TreeList_purchase;
	GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;

#if (defined DEBUG_LEVEL_1)
	printf("xcave_create_TreeList_purchase:B:%p:\n",(void *)StoreList_purchase);
#endif
	scrolledwindow_purchase = lookup_widget(window_detail,"scrolledwindow_purchase");

	/* Creates a new tree store as with n_columns columns each of the types passed in. */
	StoreList_purchase = gtk_list_store_new(NUM_COLS_TREELIST_PURCHASE, 
											G_TYPE_UINT, 
											G_TYPE_UINT, 
											G_TYPE_STRING, 
											G_TYPE_STRING, 
											G_TYPE_STRING);

	/* create view TreeList_purchase */
	TreeList_purchase = gtk_tree_view_new_with_model (GTK_TREE_MODEL(StoreList_purchase));
	gtk_widget_show (TreeList_purchase);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_purchase), TreeList_purchase);

	/* --- TreeList Column #1 --- */
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes(_("Purchase years"),
													renderer,
													"text", 
													COL_TREELIST_PURCHASE_DATE,
													NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_purchase), col);
	/* --- TreeList Column #2 -- QUANTITY --- */
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes(_("Quantity"),
												   renderer,
												   "text", 
												   COL_TREELIST_PURCHASE_QTY,
												   NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_purchase), col);
	/* --- TreeList Column #3 -- PRICE --- */
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes(_("Price"),
												   renderer,
												   "text", 
												   COL_TREELIST_PURCHASE_PRICE,
												   NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_purchase), col);
	/* --- TreeList Column #4 -- SUPPLIER --- */
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes(_("Supplier"),
												   renderer,
												   "text", 
												   COL_TREELIST_SUPPLIER,
												   NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_purchase), col);
	/* --- TreeList Column #5 -- SUPPLIER ADDRESS --- */
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes(_("Supplier address"),
												   renderer,
												   "text", 
												   COL_TREELIST_SUPPLIER_ADR,
												   NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_purchase), col);
	
	GLADE_HOOKUP_OBJECT (window_detail, TreeList_purchase, "treelist_purchase");

#if (defined DEBUG_LEVEL_1)
	printf("xcave_create_TreeList_purchase:E:%p:\n",(void *)StoreList_purchase);
#endif
	return;
}

void
on_PurchaseOk_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget		*TreeList_purchase;
	GtkListStore	*StoreList_purchase_model;
	GtkTreeIter 	pIter;
	Purchase		*tmpPurchase;
	char			statxt[80];
	GtkStatusbar	*statv;

#if (defined DEBUG_LEVEL_1)
	printf("on_PurchaseOk_clicked:B:\n");
#endif
	tmpPurchase = malloc (sizeof(Purchase));

	memset(tmpPurchase,'\0',sizeof(Purchase));

	strcpy(tmpPurchase->Supplier,
		 gtk_entry_get_text (GTK_ENTRY (lookup_widget(window_detail, "entry_purchase_supplier"))));
	strcpy(tmpPurchase->SupplierAdr,
		 gtk_entry_get_text (GTK_ENTRY (lookup_widget(window_detail, "entry_purchase_supplier_adr"))));
	tmpPurchase->Achat = atoi(
		gtk_entry_get_text (GTK_ENTRY (lookup_widget(window_detail, "entry_purchase_year"))));
	tmpPurchase->Prix = atof(
		gtk_entry_get_text (GTK_ENTRY (lookup_widget(window_detail, "spinbutton_purchase_price"))));
	tmpPurchase->qty = atoi(
		gtk_entry_get_text (GTK_ENTRY (lookup_widget(window_detail, "spinbutton_purchase_qty"))));

	gchar	tmpPrice[12];
	sprintf(tmpPrice,"%9.2f",(gfloat)tmpPurchase->Prix);

	TreeList_purchase = lookup_widget(window_detail,"treelist_purchase");
	StoreList_purchase_model = (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)TreeList_purchase);
	gtk_list_store_append(StoreList_purchase_model, &pIter);/* C */
	gtk_list_store_set(StoreList_purchase_model, &pIter,/* C */
						COL_TREELIST_PURCHASE_DATE, (guint)tmpPurchase->Achat,
						COL_TREELIST_PURCHASE_QTY, (guint)tmpPurchase->qty,
						COL_TREELIST_PURCHASE_PRICE, &tmpPrice,
						COL_TREELIST_SUPPLIER, &tmpPurchase->Supplier,
						COL_TREELIST_SUPPLIER_ADR, &tmpPurchase->SupplierAdr,
								-1);			

	NbBottleDetail += tmpPurchase->qty;

	char          ach[purchaseDate_size];
	sprintf (ach, "%d", nowan);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_year")),(gchar *) ach);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_price")), (gfloat) 0);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_qty")), (guint) 1);
	gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_supplier")),"");
	gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_supplier_adr")),"");
	
	/*
	* update de la statusbar indiquant le nb de bouteille du vin selectionne
	* se trouvant dans le stock
	*/
	statv = (GtkStatusbar *) lookup_widget (window_detail, "StatusbarDetail");
	sprintf (statxt, _("Number of bottles for the selection : %d"), NbBottleDetail);
	if (statid_detail != 0) gtk_statusbar_remove (statv, ctxtid_detail, statid_detail);
	statid_detail = gtk_statusbar_push (statv, ctxtid_detail, statxt);
	
	g_free(tmpPurchase);

#if (defined DEBUG_LEVEL_1)
	printf("on_PurchaseOk_clicked:E:\n");
#endif
}

gboolean
on_TreeList_Purchase_double_click          (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;

}
