/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "callbacks.h"
#include "interface.h"
#include "support.h"

/*
 * And the code needed to parse it
 */
static xmlPurchasePtr
parsePurchase(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
    xmlPurchasePtr ret = NULL;

    /*
     * allocate the struct
     */
    ret = (xmlPurchasePtr) malloc(sizeof(xmlPurchase));
    if (ret == NULL) {
        fprintf(stderr,"out of memory\n");
	return(NULL);
    }

	memset(ret, 0, sizeof(xmlPurchase));
	
	if ((!xmlStrcmp(cur->name, (const xmlChar *)"purchase")) && (cur->ns == ns)) {
		ret->date         = xmlGetProp(cur, (const xmlChar *) "date");
		ret->qty          = xmlGetProp(cur, (const xmlChar *) "qty");
		ret->price        = xmlGetProp(cur, (const xmlChar *) "price");
		ret->supplier     = xmlGetProp(cur, (const xmlChar *) "supplier");
		ret->supplierAdr  = xmlGetProp(cur, (const xmlChar *) "supplierAdr");
	}

    return(ret);
}

/*
 * and to print it
 */
static void
printPurchase(xmlPurchasePtr cur) {
    if (cur == NULL) return;
    if (cur->date)    printf("	date       : %s\n", cur->date);
    if (cur->qty)     printf("	qty        : %s\n", cur->qty);
    if (cur->price)   printf("	price      : %s\n", cur->price);
    if (cur->supplier)   printf("	supplier   : %s\n", cur->supplier);
    if (cur->supplierAdr) printf("	supplierAdr: %s\n", cur->supplierAdr);
    printf("\n");
}

/*
 * And the code needed to parse it
 */
static xmlDrinkingPtr
parseDrinking(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
    xmlDrinkingPtr ret = NULL;

    /*
     * allocate the struct
     */
    ret = (xmlDrinkingPtr) malloc(sizeof(xmlDrinking));
    if (ret == NULL) {
        fprintf(stderr,"out of memory\n");
	return(NULL);
    }
    memset(ret, 0, sizeof(xmlDrinking));

    if ((!xmlStrcmp(cur->name, (const xmlChar *)"drinking")) && (cur->ns == ns)) {
		ret->date       = xmlGetProp(cur, (const xmlChar *) "date");
		ret->qty        = xmlGetProp(cur, (const xmlChar *) "qty");
		ret->rated		= xmlGetProp(cur, (const xmlChar *) "rated");
		ret->comment	= xmlGetProp(cur, (const xmlChar *) "comment");
	}
    return(ret);
}

/*
 * and to print it
 */
static void
printDrinking(xmlDrinkingPtr cur) {
    if (cur == NULL) return;
    if (cur->date)    printf("	date   : %s\n", cur->date);
    if (cur->qty)     printf("	qty    : %s\n", cur->qty);
    if (cur->rated)   printf("	rated  : %s\n", cur->rated);
    if (cur->comment) printf("	comment: %s\n", cur->comment);
    printf("\n");
}


/*
 * the code needed to parse a xmlRack
 */
static xmlRacksPtr
parseRacks(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
    xmlRacksPtr ret = NULL;

    /*
     * allocate the struct
     */
    ret = (xmlRacksPtr) malloc(sizeof(xmlRacks));
    if (ret == NULL) {
        fprintf(stderr,"out of memory\n");
	return(NULL);
    }
    memset(ret, 0, sizeof(xmlRacks));

    if ((!xmlStrcmp(cur->name, (const xmlChar *)"rack")) && (cur->ns == ns)) {
		ret->rackNb = xmlGetProp(cur, (const xmlChar *) "rackNb");
		if (ret->rackNb == NULL) fprintf(stderr, "location has no rackNb\n");
		ret->row = xmlGetProp(cur, (const xmlChar *) "row");
		if (ret->row == NULL)	fprintf(stderr, "location has no row\n");
		ret->column = xmlGetProp(cur, (const xmlChar *) "column");
		if (ret->column == NULL) fprintf(stderr, "location has no column\n");
	}
    return(ret);
}

/*
 * and to print it
 */
static void
printRacks(xmlRacksPtr cur) {
    if (cur == NULL) return;
    if (cur->rackNb) printf("	rackNb  : %s\n", cur->rackNb);
    if (cur->row)    printf("	row     : %s\n", cur->row);
    if (cur->column) printf("	column  : %s\n", cur->column);
    printf("\n");
}

/*
 * the code need to print a wine
 */
void
printWine(xmlWinePtr cur) {
    int i;

    if (cur == NULL) return;
    printf("=======  Wine\n");
    if (cur->country != NULL)                 	printf("country                 : %s\n", cur->country);
    if (cur->region != NULL)                  	printf("region                  : %s\n", cur->region);
    if (cur->appellation != NULL)             	printf("appellation             : %s\n", cur->appellation);
    if (cur->domain != NULL)                  	printf("domain                  : %s\n", cur->domain);
    if (cur->vintage != NULL)                 	printf("vintage                 : %s\n", cur->vintage);
    if (cur->color != NULL)                   	printf("color                   : %s\n", cur->color);
    if (cur->wineMaker != NULL)               	printf("wineMaker               : %s\n", cur->wineMaker);
    if (cur->wineMakerAdr != NULL)            	printf("wineMakerAdr            : %s\n", cur->wineMakerAdr);
    if (cur->cepage != NULL)                  	printf("cepage                  : %s\n", cur->cepage);
    if (cur->potentielDeGarde != NULL)        	printf("potentielDeGarde        : %s\n", cur->potentielDeGarde);
    if (cur->temperatureDeDegustation != NULL)	printf("temperatureDeDegustation: %s\n", cur->temperatureDeDegustation);
    printf("%d xmlRacks\n", cur->nbRacks);
    for (i = 0;i < cur->nbRacks;i++) printRacks(cur->xmlRacks[i]);
	printf("%d xmlPurchase\n", cur->nbPurchase);
    for (i = 0;i < cur->nbPurchase;i++) printPurchase(cur->xmlPurchase[i]);
    printf("%d xmlDrinking\n", cur->nbDrinking);
    for (i = 0;i < cur->nbDrinking;i++) printDrinking(cur->xmlDrinking[i]);
    if (cur->comment != NULL)					printf("commentaire				: %s\n", cur->comment);
    if (cur->rated != NULL)						printf("rated					: %s\n", cur->rated);
    printf("======= \n");
}

/*
 * the code needed to parse a wine
 */
static xmlWinePtr
parseWine(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
    xmlWinePtr	ret = NULL;

    /*
     * allocate the struct
     */
    ret = (xmlWinePtr) malloc(sizeof(xmlWine));

    if (ret == NULL) {
        fprintf(stderr,"out of memory\n");
	return(NULL);
    }
    memset(ret, 0, sizeof(xmlWine));

    ret->nbRacks    = 0;
    ret->nbPurchase = 0;
    ret->nbDrinking = 0;

    /* We don't care what the top level element name is */
    cur = cur->xmlChildrenNode;
    while (cur != NULL) {
        if ((!xmlStrcmp(cur->name, (const xmlChar *) "country")) && (cur->ns == ns))
			ret->country = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "region")) && (cur->ns == ns))
			ret->region = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "appellation")) && (cur->ns == ns))
			ret->appellation = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "domain")) && (cur->ns == ns))
			ret->domain = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "vintage")) && (cur->ns == ns))
			ret->vintage = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "color")) && (cur->ns == ns))
			ret->color = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "wineMaker")) && (cur->ns == ns))
			ret->wineMaker = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "wineMakerAdr")) && (cur->ns == ns))
			ret->wineMakerAdr = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "cepage")) && (cur->ns == ns))
			ret->cepage = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "potentielDeGarde")) && (cur->ns == ns))
			ret->potentielDeGarde = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "temperatureDeDegustation")) && (cur->ns == ns))
			ret->temperatureDeDegustation = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "rack")) && (cur->ns == ns))
			ret->xmlRacks[ret->nbRacks++] = parseRacks(doc, ns, cur);
        if ((!xmlStrcmp(cur->name, (const xmlChar *) "purchase")) && (cur->ns == ns))
			ret->xmlPurchase[ret->nbPurchase++] = parsePurchase(doc, ns, cur);
        if ((!xmlStrcmp(cur->name, (const xmlChar *) "drinking")) && (cur->ns == ns))
			ret->xmlDrinking[ret->nbDrinking++] = parseDrinking(doc, ns, cur);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "comment")) && (cur->ns == ns))
			ret->comment = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "rated")) && (cur->ns == ns))
			ret->rated = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		cur = cur->next;
    }

	/* 
	 * xml format to list_stock format expected later
	 */
	ptrVin            tmp_pVin;
    xmlChar		*tmp;
	
	if ( ( tmp_pVin = g_malloc (sizeof (Vin))) == NULL ) {
	   fprintf (stderr, "Probleme allocation memoire.\n");
	   exit (-1);
	}

	memset(tmp_pVin,       '\0',  sizeof (Vin));

    tmp = xcave_isolat1toUtf8(ret->country);
    if (tmp != NULL) {
		strncpy (tmp_pVin->country,	tmp, country_size);
		xmlFree(tmp);
	}

    tmp = xcave_isolat1toUtf8(ret->region);
    if (tmp != NULL) {
		strncpy (tmp_pVin->Region,	tmp, region_size);
		xmlFree(tmp);
	}

    tmp = xcave_isolat1toUtf8(ret->appellation);
    if (tmp != NULL) {
		strncpy (tmp_pVin->Appellation,	tmp, appellation_size);                   
		xmlFree(tmp);
	}

    tmp = xcave_isolat1toUtf8(ret->domain);
    if (tmp != NULL) {
		strncpy (tmp_pVin->Nom, tmp, domain_size);                   
		xmlFree(tmp);
	}


	tmp_pVin->vintage = atoi(ret->vintage);             

    tmp = xcave_isolat1toUtf8(ret->color);
    if (tmp != NULL) {
		strncpy (tmp_pVin->color, tmp, color_size);
		xmlFree(tmp);
	}
	
    tmp = xcave_isolat1toUtf8(ret->wineMaker);
    if (tmp != NULL) {
		strncpy (tmp_pVin->supplier, tmp,	supplier_size);
		xmlFree(tmp);
	}

    tmp = xcave_isolat1toUtf8(ret->wineMakerAdr);
    if (tmp != NULL) {
		strncpy (tmp_pVin->supplierURL, tmp, supplierAdr_size); 
		xmlFree(tmp);
	}

    tmp = xcave_isolat1toUtf8(ret->cepage);
    if (tmp != NULL) {
		strncpy (tmp_pVin->Cepage, tmp, cepage_size);
		xmlFree(tmp);
	}

    tmp_pVin->Conserv = atoi(ret->potentielDeGarde);

    tmp = xcave_isolat1toUtf8(ret->temperatureDeDegustation);
    if (tmp != NULL) {
		strncpy (tmp_pVin->temperatureDeDegustation, tmp, temperature_size);
		xmlFree(tmp);
	}

	gint i;

	/* 
	 * update purchase list
	 */
	for ( i = 0 ; i < ret->nbPurchase ; i++) {
		ptrPurchase  tmp_pPurchase;
		if ( ( tmp_pPurchase = g_malloc (sizeof (Purchase))) == NULL ) {
		   fprintf (stderr, "Probleme allocation memoire.\n");
		   exit (-1);
		}
	
		memset(tmp_pPurchase, '\0',  sizeof (Purchase));
	
		tmp_pPurchase->Achat = atoi(ret->xmlPurchase[i]->date);
		tmp_pPurchase->qty   = atoi(ret->xmlPurchase[i]->qty);
		tmp_pPurchase->Prix  = atof(ret->xmlPurchase[i]->price);

		tmp = xcave_isolat1toUtf8(ret->xmlPurchase[i]->supplier);
		if (tmp != NULL) {
			strncpy (tmp_pPurchase->Supplier, tmp, supplier_size);                   
			xmlFree(tmp);
		}

		tmp = xcave_isolat1toUtf8(ret->xmlPurchase[i]->supplierAdr);
		if (tmp != NULL) {
			strncpy (tmp_pPurchase->SupplierAdr, tmp, supplierAdr_size);                   
			xmlFree(tmp);
		}

		tmp_pVin->NbPurchase += tmp_pPurchase->qty;
	    tmp_pVin->list_purchase = g_list_append(tmp_pVin->list_purchase,tmp_pPurchase);
	}
	
	/*
	 * update drinking list
	 */
	for ( i = 0 ; i < ret->nbDrinking ; i++) {
		ptrDrinking  tmp_pDrinking;
		if ( ( tmp_pDrinking = g_malloc (sizeof (Drinking))) == NULL ) {
		   fprintf (stderr, "Probleme allocation memoire.\n");
		   exit (-1);
		}
	
		memset(tmp_pDrinking, '\0',  sizeof (Drinking));
	
		tmp_pDrinking->Conso = atoi(ret->xmlDrinking[i]->date);
		tmp_pDrinking->qty   = atoi(ret->xmlDrinking[i]->qty);

		tmp = xcave_isolat1toUtf8(ret->xmlDrinking[i]->rated);
		if (tmp != NULL) {
			strncpy (tmp_pDrinking->Note, tmp, rated_size);                   
			xmlFree(tmp);
		}

		tmp = xcave_isolat1toUtf8(ret->xmlDrinking[i]->comment);
		if (tmp != NULL) {
			strncpy (tmp_pDrinking->Commentaire, tmp, drinkingComment_size);                   
			xmlFree(tmp);
		}

		tmp_pVin->NbDrinking += tmp_pDrinking->qty;
		
	    tmp_pVin->list_drinking = g_list_append(tmp_pVin->list_drinking,tmp_pDrinking);
	}
	tmp_pVin->qty = tmp_pVin->NbPurchase - tmp_pVin->NbDrinking;

	/*
	 * update rack list
	 */
	for ( i = 0 ; i < ret->nbRacks ; i++) {
		ptrEmplacement  tmp_pRacks;
		if ( ( tmp_pRacks = g_malloc (sizeof (Emplacement))) == NULL ) {
		   fprintf (stderr, "Probleme allocation memoire.\n");
		   exit (-1);
		}
	
		memset(tmp_pRacks, '\0',  sizeof (Emplacement));
	
		tmp_pRacks->Rack	= atoi(ret->xmlRacks[i]->rackNb);
		tmp_pRacks->Row		= atoi(ret->xmlRacks[i]->row);
		tmp_pRacks->Column	= atoi(ret->xmlRacks[i]->column);
	
		strncpy (tmp_pRacks->Nom, tmp_pVin->Nom, domain_size);                   

	    tmp_pVin->list_emplacement = g_list_append(tmp_pVin->list_emplacement,tmp_pRacks);
	}

    tmp = xcave_isolat1toUtf8(ret->rated);
    if (tmp != NULL) {
		strncpy (tmp_pVin->Note, tmp, rated_size);
		xmlFree(tmp);
	}

    tmp = xcave_isolat1toUtf8(ret->comment);
    if (tmp != NULL) {
		strncpy (tmp_pVin->Commentaire, tmp, comment_size);
		xmlFree(tmp);
	}

    list_stock = g_list_append(list_stock ,tmp_pVin);

#if (defined DEBUG_LEVEL_2)
	printWine(ret);
#endif

    return(ret);
}

static xmlWinePtr
parseXmlWineFile(char *filename) {
    xmlDocPtr	doc;
	xmlWinePtr	curXmlWine;
    xmlNsPtr	ns;
    xmlNodePtr	cur;

#ifdef LIBXML_SAX1_ENABLED
    /*
     * build an XML tree from a the file;
     */
    doc = xmlParseFile(filename);
    if (doc == NULL) return(NULL);
#else
    /*
     * the library has been compiled without some of the old interfaces
     */
    return(NULL);
#endif /* LIBXML_SAX1_ENABLED */

    /*
     * Check the document is of the right kind
     */
    
    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {
        fprintf(stderr,"empty document\n");
	xmlFreeDoc(doc);
	return(NULL);
    }
	ns = xmlSearchNsByHref(doc, cur,(const xmlChar *) "http://xcave.free.fr/xmlwine");
    if (ns == NULL) {
        fprintf(stderr,"document of the wrong type, xcave Namespace not found\n");
		xmlFreeDoc(doc);
		return(NULL);
    }
    if (xmlStrcmp(cur->name, (const xmlChar *) "xcave")) {
        fprintf(stderr,"document of the wrong type, root node != Helping");
		xmlFreeDoc(doc);
		return(NULL);
    }

    /*
     * Allocate the structure to be returned.
     */
    curXmlWine = (xmlWinePtr) malloc(sizeof(xmlWine));
    if (curXmlWine == NULL) {
        fprintf(stderr,"out of memory\n");
	xmlFreeDoc(doc);
	return(NULL);
    }
    memset(curXmlWine, 0, sizeof(xmlWine));

    /*
     * Now, walk the tree.
     */
    /* First level we expect just Wines */
    cur = cur->xmlChildrenNode;
    while ( cur && xmlIsBlankNode ( cur ) ){
		cur = cur -> next;
	}
    if ( cur == 0 )      return ( NULL );
		
    if ((xmlStrcmp(cur->name, (const xmlChar *) "wines")) || (cur->ns != ns)) {
        fprintf(stderr,"document of the wrong type, was '%s', wines expected ",cur->name);
		fprintf(stderr,"xmlDocDump follows\n");
#ifdef LIBXML_OUTPUT_ENABLED
		xmlDocDump ( stderr, doc );
		fprintf(stderr,"xmlDocDump finished\n");
#endif /* LIBXML_OUTPUT_ENABLED */
		xmlFreeDoc(doc);
		free(curXmlWine);
		return(NULL);
    }

    /* Second level is a list of Wine */
    cur = cur->xmlChildrenNode;
    while (cur != NULL) {
        if ((!xmlStrcmp(cur->name, (const xmlChar *) "wine")) && (cur->ns == ns)) {
			curXmlWine = parseWine(doc, ns, cur);
		}
		cur = cur->next;
    }

    return(curXmlWine);
}

int
xcave_xmlread(void) {

	LIBXML_TEST_VERSION
    xmlKeepBlanksDefault(0);

	if ( parseXmlWineFile((char *)DBPATH) == NULL ) 
		return(-1);

    /* Clean up everything else before quitting. */
    xmlCleanupParser();

    return(0);
}

xmlChar *
xcave_isolat1toUtf8(const char * utf8Tag) {

	int ret = 0;

	if ( utf8Tag == NULL ) return(NULL);

	int utf8TagLength = strlen(utf8Tag);
	int isolat1TagLength = 2*strlen(utf8Tag);
	xmlChar *isolat1Tag  = xmlMalloc( isolat1TagLength * sizeof(isolat1Tag));
	memset(isolat1Tag,'\0',isolat1TagLength * sizeof(isolat1Tag));
	
	/* Let's take the input tag and convert from an assumed format of
	 * ISO-8559-1 (iso-latin-1) to the internal representation UTF-8
	 */
	ret = UTF8Toisolat1(isolat1Tag, &isolat1TagLength, utf8Tag, &utf8TagLength);
	if (ret != -1){
        /* It worked, let's make sure that it processed the whole input string */
		if (utf8TagLength != strlen(utf8Tag)){
			fprintf(stderr, "UTF8Toisolat1 could not process entire TAG\n");
			xmlFree(isolat1Tag);
			isolat1Tag = NULL;
		}
	}
	else {
		fprintf(stderr, "UTF8Toisolat1 returned an error : %d %d !!\n",ret,errno);
		xmlFree(isolat1Tag);
        isolat1Tag = NULL;
	}
	return(isolat1Tag);
}
